var Localization = {
    getRows: function() {
        var rows = [];
        try {
            var list = this.getModelProperty("MetaData/:demo/List");
            if (typeof list === 'string') {
                rows = JSON.parse(list);
            } else if (Array.isArray(list)) {
                rows = list;
            }
        } catch (e) {
            rows = [];
        }
        return rows;
    },

    getCols: function() {
        var cols = [];
        var config = this.getConfig();
        Object.keys(config).forEach(function(key) {
            if (key === 'DataLayoutID' || config[key] === '/') {
                return;
            }
            cols.push(config[key].replace('/', ':'));
        });

        return cols;
    },

    initModel: function() {
        this.model = {};
        this.elems = {};
        this.model.rows = this.getRows();
        this.model.cols = this.getCols();
    },

    buildComponent: function() {
        this.initModel();

        // Container (div that contains the whole plugin)
        var $container = document.createElement('div');
        $container.className = 'dynamic-array-plugin';

        // Button to add a row
        var $addRowButton = document.createElement('button');
        $addRowButton.textContent = this.getTranslation('addRow', 'Add row');
        $container.appendChild($addRowButton);

        // Table
        var $table = this.createTable();
        $container.appendChild($table);

        // Click event on button
        var that = this;
        $addRowButton.addEventListener('click', function() {
            that.model.rows.push({});
            $table.appendChild(this.addRow(that.model.rows.length - 1));
        }.bind(this));

        return $container;
    },

    update: function() {
        this.model.rows = this.getRows();
        this.fillTable();
    },

    createTable: function() {
        var $table = document.createElement('table');
        this.elems.$table = $table;

        var $header = document.createElement('tr');
        this.model.cols.forEach(function(value, index) {
            var $th = document.createElement('th');
            $th.textContent = value;
            $header.appendChild($th);
        }, this);
        $header.appendChild(document.createElement('th'));
        $table.appendChild($header);

        this.fillTable();

        return $table;
    },

    fillTable: function() {
        // Empty the table, except for the header row
        while (this.elems.$table.childElementCount > 1) {
            this.elems.$table.removeChild(this.elems.$table.lastChild);
        }

        // Add rows
        this.model.rows.forEach(function(row, index) {
            this.addRow(index);
        }, this);
    },

    addRow: function(index) {
        // Row
        var $row = document.createElement('tr');

        var that = this;
        this.model.cols.forEach(function(col) {
            var $col = document.createElement('td');
            var $input = document.createElement('input');
            $input.placeholder = 'Set a value';
            $input.value = that.model.rows[index][col] || '';

            $input.addEventListener('change', function() {
                that.model.rows[index][col] = $input.value;
                that.setModelProperty("MetaData/:demo/List", JSON.stringify(that.model.rows));
            });

            $col.appendChild($input);
            $row.appendChild($col);
        });
        this.elems.$table.appendChild($row);

        // Button remove
        var $col = document.createElement('td');
        var removeRowButton = document.createElement('button');
        removeRowButton.textContent = this.getTranslation('remove', 'Remove');
        removeRowButton.onclick = function() {
            that.model.rows.splice(index, 1);
            that.setModelProperty("MetaData/:demo/List", JSON.stringify(that.model.rows));
        }.bind(this);
        $col.appendChild(removeRowButton);
        $row.appendChild($col);

        return $row;
    },

    getConfigSetup: function() {
        return {
            Metadata1: {
                type: 'MetadataState',
                translation: this.getTranslation('Metadata', 'Metadata') + ' 1',
                metadataType: ['String'],
            },
            Metadata2: {
                type: 'MetadataState',
                translation: this.getTranslation('Metadata', 'Metadata') + ' 2',
                metadataType: ['String'],
            },
            Metadata3: {
                type: 'MetadataState',
                translation: this.getTranslation('Metadata', 'Metadata') + ' 3',
                metadataType: ['String'],
            },
            Metadata4: {
                type: 'MetadataState',
                translation: this.getTranslation('Metadata', 'Metadata') + ' 4',
                metadataType: ['String'],
            },
            Metadata5: {
                type: 'MetadataState',
                translation: this.getTranslation('Metadata', 'Metadata') + ' 5',
                metadataType: ['String'],
            },
            Metadata6: {
                type: 'MetadataState',
                translation: this.getTranslation('Metadata', 'Metadata') + ' 6',
                metadataType: ['String'],
            },
        };
    },
};